package DataStructure::LIFO;

use strict;
use warnings;
use utf8;
use feature ':5.24';
use feature 'signatures';
no warnings 'experimental::signatures';

# Empty, only a declaration of a role that can be used by other data-structures.
# In the future this might be a Role::Tiny role.

# In addition, this is a synonym for Stack, so we’re not defining additional
# synonyms here.

1;
